signature ENV =
sig
  type access
  type level
  type label
  type ty

  datatype enventry 
    = VARentry of {access: access, ty: ty}
    | FUNentry of {level: level, label: label, formals: ty list, result: ty}

  type tenv = ty Symbol.table
  type env = enventry Symbol.table

  val base_tenv : tenv
  val base_env : env
end

functor EnvGen(Translate: TRANSLATE) : ENV =

struct

  structure S = Symbol
  structure T = Types

  type access = Translate.access
  type level = Translate.level
  type label = Temp.label
  type ty = T.ty

  datatype enventry 
    = VARentry of {access: access, ty: ty}
    | FUNentry of {level: level, label: label, formals: ty list, result: ty}

  type tenv = ty Symbol.table

  type env = enventry Symbol.table

  fun update ((sym,ent),env) = S.enter (env, S.symbol sym, ent)

  (* here you need to add all primtive types into the base_tenv *)
  val base_tenv : tenv = List.foldl update S.empty [("int", T.INT),
						    ("string", T.STRING)]

  fun mkstdfun name formals result =
      (name, FUNentry {level=Translate.outermost, 
                       label=Temp.namedlabel name, 
                       formals=formals, 
                       result=result})
 
  (* here you need to add all primitive library functions into the base_env *)
  val base_env : env = List.foldl update S.empty
		       (List.map 
			    (fn (n,t,r) => mkstdfun n t r) 
			    [("print", [T.STRING], T.UNIT),
			     ("flush", [], T.UNIT),
			     ("getchar", [], T.STRING),
			     ("ord", [T.STRING], T.INT),
			     ("chr", [T.INT], T.STRING),
			     ("size", [T.STRING], T.INT),
			     ("substring", [T.STRING, T.INT, T.INT], T.STRING),
			     ("concat", [T.STRING, T.STRING], T.STRING),
			     ("not", [T.INT], T.INT),
			     ("exit", [T.INT], T.UNIT)])

end  (* structure Env *)
  
