(* frame.sml *)

signature FRAME =
sig 
  type offset = int
  type frame
  
  val newFrame : int -> frame * offset list
  val allocInFrame : frame -> offset

  datatype frag = PROC of {name : Temp.label, body: Tree.stm, frame: frame} 
                | DATA of {lab : Temp.label, s: string}

  val print_frame : frame -> unit

end (* signature FRAME *)


structure Frame : FRAME = 
struct
  type offset = int
  type frame = {formals : int,         (* number of formal parameters *)
                offlst : offset list,  (* offset list for formals *)
                locals : int ref,      (* # of local variables so far *)
                maxargs : int ref}     (* max outgoing args for the function *)

  fun print_frame {formals, offlst, locals, maxargs} = 
      (print "frame: ";
      print ("{formals = " ^ Int.toString formals);
      print (", locals = " ^ Int.toString (!locals));
      print (", maxargs = " ^ Int.toString (!maxargs));
      print "}"
      )


  datatype frag = PROC of {name : Temp.label, body: Tree.stm, frame: frame} 
                | DATA of {lab : Temp.label, s: string}

  local exception NotImplemented
  in
  fun newFrame n =
      let val offsetlist = List.tabulate (n, (fn x => Register.paramBaseOffset + 4*x)) in
      ({formals=n, offlst = offsetlist,
        locals = ref 0, maxargs = ref 0}, offsetlist)
      end


  fun allocInFrame {formals, offlst,locals, maxargs} = 
      (locals := !locals +1; !locals * ~4 + Register.localsBaseOffset)
      
  end

end (* structure Frame *)

